package fragment;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.util.Pair;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.fragment.app.Fragment;

import com.akamai.amp.captioning.AmpCaptionComponent;
import com.akamai.amp.exoplayer2.util.MimeTypes;
import com.akamai.captioning.ampcaptioningsample.MainActivity;
import com.akamai.captioning.ampcaptioningsample.R;
import com.akamai.amp.media.IPlayerEventsListener;
import com.akamai.amp.media.VideoPlayerContainer;
import com.akamai.amp.media.VideoPlayerView;
import com.akamai.amp.media.elements.MediaResource;
import com.akamai.amp.media.errors.ErrorType;
import com.akamai.amp.media.exowrapper2.SideloadedCaptionsData;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

public class WebVTTFragment extends Fragment
        implements View.OnTouchListener, IPlayerEventsListener {

    private final static String TAG = "AMP-CC-WEBVTT";
    private final static String VIDEO_URL = "https://multiplatform-f.akamaihd.net/i/multi/will/bunny/big_buck_bunny_,640x360_400,640x360_700,640x360_1000,950x540_1500,.f4v.csmil/master.m3u8";

    private static final String VTT_CAPTIONS = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/sample.vtt";

    private static final String TTML_CAPTIONS = "http://projects.mediadev.edgesuite.net/jchavarr/assets/ttl-w3-sample1.ttml";

    private static final String HLS_WITH_VTTS = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/vtt-sideload-playlist-min.m3u8";

    private static final String WEBVTTS_A = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/a.vtt";
    private static final String WEBVTTS_B = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/b.vtt";
    private static final String WEBVTTS_C = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/c.vtt";
    private static final String WEBVTTS_D = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/d.vtt";

    private static final String WEBVTTS_LIST_A = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/vtt-sideload-playlista.m3u8";
    private static final String WEBVTTS_LIST_B = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/vtt-sideload-playlistb.m3u8";
    private static final String WEBVTTS_LIST_C = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/vtt-sideload-playlistc.m3u8";
    private static final String WEBVTTS_LIST_D = "http://projects.mediadev.edgesuite.net/customers/akamai/android/doc/vtt-sideload-playlistd.m3u8";


    private VideoPlayerContainer mVideoContainer;
    private VideoPlayerView mVideoPlayerView;
    private ProgressBar mProgressBar;
    private AmpCaptionComponent captionComponent;

    private Button showCaptionsButton;
    private Button hideCaptionsButton;
    private Button nextCaptionsButton;


    public static WebVTTFragment newInstance() {
        WebVTTFragment fragment = new WebVTTFragment();
        return fragment;
    }

    public WebVTTFragment() {
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        View rootView = inflater.inflate(R.layout.fragment_main, container, false);
        mProgressBar = rootView.findViewById(R.id.progressBar);

        captionComponent = rootView.findViewById(R.id.captionView);

        mVideoContainer = rootView.findViewById(R.id.playerViewCtrl);
        mVideoContainer.addVideoPlayerContainerCallback(new VideoPlayerContainer.VideoPlayerContainerCallback() {
            @Override
            public void onVideoPlayerCreated() {
                Log.i(TAG, "onVideoPlayerCreated()");
            }

            @Override
            public void onResourceReady(MediaResource mediaResource) {
                mVideoPlayerView = mVideoContainer.getVideoPlayer();
                mVideoPlayerView.setProgressBarControl(mProgressBar);
                mVideoPlayerView.setLicense(MainActivity.LICENSE);

                mVideoPlayerView.setFullScreen(true);
                mVideoPlayerView.setFullScreenMode(VideoPlayerView.FULLSCREEN_MODE_KEEP_ASPECT_RATIO);
                mVideoPlayerView.setLogEnabled(true);

                initCaptions();

                mVideoPlayerView.addEventsListener(WebVTTFragment.this);
                mVideoPlayerView.setKeepScreenOn(true);
                mVideoPlayerView.setOnTouchListener(WebVTTFragment.this);

                mVideoPlayerView.play(mediaResource);
            }

            @Override
            public void onResourceError(ErrorType errorType, Exception exception) {

                Log.e(TAG, "onResourceError()");
            }
        });

        showCaptionsButton = rootView.findViewById(R.id.showCaptionsButton);
        showCaptionsButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                captionComponent.show();
                Toast.makeText(getContext(), "Showing captions", Toast.LENGTH_SHORT).show();
            }
        });
        showCaptionsButton.setVisibility(View.VISIBLE);

        hideCaptionsButton = rootView.findViewById(R.id.hideCaptionsButton);
        hideCaptionsButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                captionComponent.hide();
                Toast.makeText(getContext(), "Hiding captions", Toast.LENGTH_SHORT).show();
            }
        });
        hideCaptionsButton.setVisibility(View.VISIBLE);

        nextCaptionsButton = rootView.findViewById(R.id.nextCaptionsButton);
        nextCaptionsButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int captionsID = loadNextCaptions();
                Toast.makeText(getContext(), "Loading captions #" + captionsID, Toast.LENGTH_SHORT).show();
            }
        });
        nextCaptionsButton.setVisibility(View.VISIBLE);

        mVideoContainer.prepareResource(VIDEO_URL);

        return rootView;
    }

    /**
     * Caption configuration
     */
    private void initCaptions(){
        captionComponent.setVideoPlayerView(mVideoPlayerView);

        //SideloadedCaptionsData cc0 = new SideloadedCaptionsData(MimeTypes.TEXT_VTT, VTT_CAPTIONS);
        //SideloadedCaptionsData cc1 = new SideloadedCaptionsData(MimeTypes.APPLICATION_TTML, TTML_CAPTIONS);
        SideloadedCaptionsData cc2 = new SideloadedCaptionsData(MimeTypes.TEXT_VTT, WEBVTTS_A, "cc-id-0", "english");
        SideloadedCaptionsData cc3 = new SideloadedCaptionsData(MimeTypes.TEXT_VTT, WEBVTTS_B, "cc-id-1", "japanese");
        SideloadedCaptionsData cc4 = new SideloadedCaptionsData(MimeTypes.TEXT_VTT, WEBVTTS_C, "cc-id-2", "french");
        SideloadedCaptionsData cc5 = new SideloadedCaptionsData(MimeTypes.TEXT_VTT, WEBVTTS_D, "cc-id-3", "klingon");

        //List<SideloadedCaptionsData> captions = Arrays.asList(cc0, cc1, cc2, cc3, cc4, cc5);
        List<SideloadedCaptionsData> captions = Arrays.asList(cc2, cc3, cc4, cc5);

        captionComponent.sideload(captions);
    }

    int nextCaptions = 0;

    private int loadNextCaptions(){
        nextCaptions++;
        Map<Pair<Integer, Integer>, String> dataTracks = captionComponent.getDataTracks();
        int size = dataTracks.size();
        if (nextCaptions >= size) {
            nextCaptions = 0;
        }

        captionComponent.setIndexes(nextCaptions, 0);

        return nextCaptions + 1;
    }

    @Override
    public void onResume() {
        Log.d(TAG, "OnResume");
        if (mVideoPlayerView != null) {
            mVideoPlayerView.resume();
        }
        super.onResume();
    }

    @Override
    public void onPause() {
        Log.d(TAG, "OnPause");
        if(mVideoPlayerView != null){
            mVideoPlayerView.pause();
        }
        super.onPause();
    }

    @Override
    public void onDestroy() {
        Log.d(TAG, "OnDestroy");
        if (mVideoPlayerView!=null) {
            mVideoPlayerView.stop();
            mVideoPlayerView.onDestroy();
        }
        super.onDestroy();
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
    }

    @Override
    public boolean onPlayerEvent(int eventID) {
        return true;
    }

    @Override
    public boolean onPlayerExtendedEvent(int eventID, int arg1, int arg2) {
        return true;
    }

    @Override
    public boolean onTouch(View v, MotionEvent event)
    {
        return false;
    }

}